/*
 * BigBroBigSis.java 1.0 
 * Copyright 2003 by Byron Weber Becker.  All rights reserved.
 */
import becker.io.TextOutput;
import becker.io.TextInput;
import java.util.Comparator;
import becker.util.DateTime;

/** A list of the "bigs" and "littles" associated with a Big Brother/Big Sister 
 * program.  "Bigs" are the Big Brothers and Big Sisters;  "littles" are 
 * the Little Brothers and Sisters they are (potentially) paired with.
 * @author Byron Weber Becker */
public class BigBroBigSis extends Object
{	
	private Person[] persons;		// the list of bigs and littles
	
	/** Construct a new object by reading all the bigs and littles from a file.
	 * @param fileName the name of the file storing the information for bigs and littles */
	public BigBroBigSis(String fileName)
	{	super();
/*		
		TextInput in = new TextInput(fileName);

		// get the number of records in the file
		int count = in.readInt();
		in.readLine();
		
		// allocate an array to hold each record we read
		this.persons = new Person[count];
		
		// Read the data, storing a reference to each object in the array
		for(int i=0; i<count; i++)
		{	this.persons[i] = new Person(in);	
		}
		in.close();
*/		
	
		// count the number of Persons in the file
		int count = 0;
		TextInput in = new TextInput(fileName);
		while(!in.eofIsAvailable())
		{	Person p = new Person(in);
			count++;
		}
		in.close();
		
		// allocate an array to hold each object we read
		this.persons = new Person[count];
		
		// Read the data, storing a reference to each object in the array
		in = new TextInput(fileName);
		for(int i=0; i<count; i++)
		{	this.persons[i] = new Person(in);	
		}
		in.close();

	}
	
	/** Search for the named person.
	 * @param name The name of the person to search for.
	 * @returns The first matching person object;  null if there is none. */
	public Person search(String name)
	{	for(int i=0; i<this.persons.length; i++)
		{	if (this.persons[i].getName().equals(name))
			{	return this.persons[i];
			}
		}	
		return null;
	}
	
	/** Find the most compatible person for p.  
	 * @param p The person to find a match for
	 * @returns The most compatible person for p; null if there isn't one 
	 * (e.g. everyone else is already matched) */
	public Person findMostCompatible(Person p)
	{	Person mostCompatibleSoFar = null;
		double compatScore = -1.0;		// less than any valid compatibility score
		for(int i=1; i<this.persons.length; i++)
		{	if (this.persons[i].getCompatibility(p) > compatScore)
			{	mostCompatibleSoFar = this.persons[i];
				compatScore = mostCompatibleSoFar.getCompatibility(p);
			}
		}
		return mostCompatibleSoFar;
	}
	
	/** Find the oldest person in the list. 
	 * (Assumes there is at least one person in the array. */
	public Person findFirstBirthday()
	{	Person oldestSoFar = this.persons[0];	// role: most-wanted holder
		DateTime oldestBirthdate = oldestSoFar.getBirthdate();	// role: follower
		
		for(int i=1; i<this.persons.length; i++)
		{	Person currentPerson = this.persons[i];	// role: temporary
			DateTime currentBirthdate = currentPerson.getBirthdate();	// role: follower
			if (currentBirthdate.isBefore(oldestBirthdate))
			{	oldestSoFar = currentPerson;
				oldestBirthdate = currentBirthdate;
			}
		}
		return oldestSoFar;
	}
	
	/** Find the oldest person in the list. 
	 * (Assumes there is at least one person in the array. */
	public Person findOldestPerson()
	{	Person oldestSoFar = this.persons[0];	// role: most-wanted holder
		
		for(int i=1; i<this.persons.length; i++)
		{	Person currentPerson = this.persons[i];	// role: temporary
			if (currentPerson.getAge() > oldestSoFar.getAge())
			{	oldestSoFar = currentPerson;
			}
		}
		return oldestSoFar;
	}
		
	/** List all the bigs and littles on System.out. */
	public void listAll()
	{	for(int i=0; i<this.persons.length; i++)
		{	System.out.println(this.persons[i].getAge() /*.toString()*/);
		}	
	}
	
	/** List, on System.out, all the compatible persons.
	 * @param p The person to be compatible with
	 * @param minCompatibility The minimum acceptable compatibility score */
	public void listCompatible(Person p, double minCompatibility)
	{	for (int i = 0; i<this.persons.length; i++)
		{	Person other = this.persons[i];
			if (other.getCompatibility(p) > minCompatibility)
			{	System.out.println(other.getCompatibility(p) + "  " + other);
			} 
		}
	}
	
	/** Write the entire list of bigs and littles to a file.
	 * @param out The already open output file. */
	public void write(TextOutput out)
	{	for (int i = 0; i<this.persons.length; i++)
		{	this.persons[i].write(out);
		}
	}
	
	/** Sort the list of persons in alphabetical order by name. */
	public void sort()
	{	for(int firstUnsorted=0; firstUnsorted<this.persons.length-1; firstUnsorted++)
		{	int extremeIndex = this.findExtreme(firstUnsorted);
			this.swap(firstUnsorted, extremeIndex);
		}
	}
	
	/** Find the extreme element in the unsorted portion of the array.
	 * @param indexToStart the smallest index in the unsorted portion of the array
	 * @return the index of the extreme element. */
	private int findExtreme(int indexToStart)
	{	int indexBestSoFar = indexToStart;
		String nameBestSoFar = this.persons[indexBestSoFar].getName();
		for(int i=indexToStart+1; i<this.persons.length; i++)
		{	String currPersonName = this.persons[i].getName();
			if (currPersonName.compareTo(nameBestSoFar) < 0)
			{	indexBestSoFar = i;
				nameBestSoFar = this.persons[i].getName();
			}
		}
		return indexBestSoFar;
	}
	
	/** Swap the elements at indices a and b. */
	private void swap(int a, int b)
	{	Person temp = this.persons[a];
		this.persons[a] = this.persons[b];
		this.persons[b] = temp;
	}
	
	
	/** Sort the persons array in increasing order by age */
	public void sortByAge()
	{	for(int firstUnsorted=0; firstUnsorted<this.persons.length-1; firstUnsorted++)
		{	// find the youngest unsorted person
			int extremeIndex = firstUnsorted;
			for(int i=firstUnsorted + 1; i<this.persons.length; i++)
			{	if (this.persons[i].getAge() < this.persons[extremeIndex].getAge())
				{	extremeIndex = i;
				}
			}
			
			// swap the youngest unsorted person with the person at firstUnsorted
			Person temp = this.persons[extremeIndex];
			this.persons[extremeIndex] = this.persons[firstUnsorted];
			this.persons[firstUnsorted] = temp;
		}
	}
	
	
	/** Sort the list of persons in alphabetical order by name. */
	/*
	public void sort()
	{	for(int firstUnsorted=0; firstUnsorted<this.persons.length-1; firstUnsorted++)
		{	// find min in unsorted part of the list
			int min = firstUnsorted;
			for(int i=firstUnsorted+1; i<this.persons.length; i++)
			{	if (this.persons[i].getName().compareTo(this.persons[min].getName()) < 0)
				{	min = i;
				}
			}
			
			// swap
			Person temp = this.persons[firstUnsorted];
			this.persons[firstUnsorted] = this.persons[min];
			this.persons[min] = temp;
		}
	}
	*/
	
	/** Sort the list of persons according to a caller-specified criteria.
	 * @param comparator An object implementing the Comparator interface
	 *    that is used to define the ordering of the person objects */
	public void sort(Comparator comparator)
	{	for(int firstUnsorted=0; firstUnsorted<this.persons.length-1; firstUnsorted++)
		{	// find min in unsorted part of the list
			int min = firstUnsorted;
			for(int i=firstUnsorted+1; i<this.persons.length; i++)
			{	if (comparator.compare(this.persons[i], this.persons[min]) < 0)
				{	min = i;
				}
			}
			
			// swap
			Person temp = this.persons[firstUnsorted];
			this.persons[firstUnsorted] = this.persons[min];
			this.persons[min] = temp;
		}
	}
}
