import becker.xtras.connect4.*;
import ljwari.io.ConsoleInput;

/** A class providing a text-based interface for a game of Connect4.
 * @author Byron Weber Becker 
 **/
class TextUI extends Object
{	private Connect4 c4;						// the game's logic
	private ConsoleInput in = new ConsoleInput();	// user input

	/** Construct a user interface object.
	 * @param theGame The game object to interface with. */
	public TextUI(Connect4 theGame)
	{	super();
		this.c4 = theGame;
	}
   
   /** Play the game by repeatedly displaying the board, asking the user for a 
    *  move, and making that move. */
   public void playGame()
   {  this.showRules();
      
      while (!this.c4.isOver())
		{	System.out.println();
			System.out.print(this.c4.getBoard());
			int column = this.getLegalMove();
			this.c4.placeToken(column);
		}
		System.out.println(this.c4.getEndOfGameMessage());
		System.out.println(this.c4.getBoard());
	}
   
 	/** Show the games rules to the user. */
   private void showRules()
   {  System.out.println(
         "   *** Connect 4 ***\n" +
         "You and the computer will take turns placing tokens\n" +
         "in the columns on the 'board'.  Tokens always fall\n" +
         "to the bottom of the column.  The first player to\n" +
         "place four tokens in a row either horizontally,\n" +
         "vertically, or diagonally is the winner.\n\n" +
         "You will play '" + Connect4.HUMAN + "' and the computer will play '" + Connect4.COMPUTER + "'.\n" +
         "To move, enter a column number between " + Connect4.FIRST_COLUMN + " and " + Connect4.LAST_COLUMN + ".\n"
         );
   }
   
   /** Get a legal move from the user (valid column that is not full).
    * @return a legal move */
   private int getLegalMove()
   {	int column = this.in.demandInt("Enter a column ", Connect4.FIRST_COLUMN, Connect4.LAST_COLUMN);
		while (!this.c4.isLegalMove(column))
		{	column = this.in.demandInt(column + " is not a legal move: ", Connect4.FIRST_COLUMN, Connect4.LAST_COLUMN);	
		}
		return column;
   }
}
