/*
 * ConsoleInput.java 1.0 
 * Copyright 2003 by Byron Weber Becker.  All rights reserved.
 */
package ljwari.io;
import becker.io.TextInput;

/** Provide useful, general purpose methods for reading from a user. */
public class ConsoleInput extends TextInput
{
	public ConsoleInput()
	{	super();
	}
	
	/** Demand an integer from the user, asking repeatedly until an integer is entered.
	 * @param firstPrompt The initial prompt given to the user
	 * @param errorPrompt The prompt given to the user if they do not answer correctly
	 * @returns the integer entered by the user */
	public int demandInt(String firstPrompt, String errorPrompt)
	{
		System.out.print(firstPrompt);
		while (!this.intIsAvailable())
		{	this.readLine();
			System.out.print(errorPrompt);
		}
		int answer = this.readInt();
		return answer;
	}
	
	/** Prompt for and demand an integer in the given range.  Keep asking
	 * until the user enters an acceptable integer.
	 * @param prompt The prompt to display to the user.
	 * @param low The lowest acceptable integer.
	 * @param high The highest acceptable integer.
	 * @return The integer entered */
   public int demandInt(String prompt, int low, int high)
   {  System.out.print(prompt + " [" + low + "-" + high + "] ");
      while (true)
      {  if (this.intIsAvailable())
         {  int i = this.readInt();
            this.readLine();
            if (i >= low && i <= high)
            {  return i;
            }
         }
         System.out.print("Please enter a number between " + low +
                          " and " + high + ": ");
      }
   }

	/** Prompt for and demand a yes or no, true or false answer.  Keep asking
	 * until the user enters an appropriate response.
	 * @param prompt The prompt to display to the user.
	 * @return true if the user answers with "true" or "yes" and false if the
	 * user answers with "false" or "no". */
	public boolean demandYesNo(String prompt)
	{	System.out.print(prompt);
	 	while (true)
	 	{	if (this.tokenIsAvailable())
	 		{	String answer = this.readLine();
	 			if (answer.equalsIgnoreCase("yes") ||
	 				 answer.equalsIgnoreCase("true"))
	 			{	return true;
	 			} else if (answer.equalsIgnoreCase("no") ||
	 				        answer.equalsIgnoreCase("false"))
	 			{	return false;
	 			} 
	 		}
	 		System.out.print("Please enter 'yes', 'no', 'true' or 'false': ");
	 	}
	}
}