import becker.robots.*;


/** A LimitedBot can carry or hold only a limited number of things.  The 
 * actual limit set when the robot is constructed.
 *
 * @author Byron Weber Becker */
public class LimitedBot extends Robot
{
	private int max;				// maximum # of things this robot can hold
	private int numHeld = 0;	// number of things currently held by this robot
	
	/** Construct a new LimitedBot
	 * @param c This robot's city
	 * @param ave This robot's initial avenue
	 * @param str This robot's initial street
	 * @param dir This robot's initial direction
	 * @param maxThings The maximum number of things this robot can carry/hold. */
	public LimitedBot(City c, int ave, int str, int dir, int maxThings)
	{	super(c, ave, str, dir);
		this.max = maxThings;
	}
	
	/** Pick up a thing.  If the robot is already holding the maximum number 
	 * of things, it breaks. */
	public void pickThing()
	{	super.pickThing();
		this.numHeld = this.numHeld + 1;
		if (this.numHeld > this.max)
		{	this.breakRobot();
		}
	}
	
	/** Put down one thing. */
	public void putThing()
	{	super.putThing();
		this.numHeld = this.numHeld - 1;
	}
}

