
import java.awt.*;
import java.awt.geom.*;
import becker.util.Utilities;
import becker.util.Test;


/** A fourth try at the Robot class.  These robots are constructed on an
 * intersection specified by the client, facing East.  Robots can move 
 * forward and turn left.  Robots are shown with the familiar arrowhead 
 * so the user can determine which way they are facing. 
 *
 * @author Byron Weber Becker */
public class SimpleBot extends Paintable
{
   private int avenue;
   private int street;
   private int direction;
   
   private static final int BODY_RADIUS = 15;
   private static final int SENSOR_RADIUS = 6;
   public static final int EAST = 0;
   public static final int SOUTH = 1;
   public static final int WEST = 2;
   public static final int NORTH = 3;

   /** Construct a new robot in the given city at the given location.
    * @param aCity the city in which this robot appears
    * @param anAvenue this robot's initial avenue
    * @param aStreet this robot's initial street 
    * @param aDir this robot's initial direction*/
   public SimpleBot(SimpleCity c, int anAvenue, int aStreet, int aDir)
   {  super();
      this.avenue = anAvenue;
      this.street = aStreet;
      this.direction = aDir;
      c.add(this, 2);
   }

   /** Paint the robot at its current location. */
   public void paint(Graphics2D g)
   {  g.setColor(Color.red);
   
      int cx = this.avenue * SimpleIntersection.SIZE
            + SimpleIntersection.SIZE / 2;
      int cy = this.street * SimpleIntersection.SIZE
            + SimpleIntersection.SIZE / 2;
      g.fillOval(cx - BODY_RADIUS, cy - BODY_RADIUS, 2 * BODY_RADIUS,
            2 * BODY_RADIUS);
      cx = cx + this.aveOffset() * BODY_RADIUS;
      cy = cy + this.strOffset() * BODY_RADIUS;
      g.fillOval(cx - SENSOR_RADIUS, cy - SENSOR_RADIUS, 2 * SENSOR_RADIUS,
            2 * SENSOR_RADIUS);
   }

   /** Move the robot forward 1 intersection. */
   public void move()
   {  this.avenue = this.avenue + this.aveOffset();
      this.street = this.street + this.strOffset();
      Utilities.sleep(400);
   }

   /** Turn the robot left 1/4 turn. */
   public void turnLeft()
   {  this.direction = this.direction + 3;
      Utilities.sleep(400);
   }

   /** Calculate how far the robot should move along the avenue
    @return {-1, 0, or 1} */
   private int aveOffset()
   {  int offset = (int) (Math.cos(this.direction * Math.PI / 2));
      return offset;
   }

   /** Calculate how far the robot should move along the street
    @return {-1, 0, or 1} */
   private int strOffset()
   {  int offset = (int) (Math.sin(this.direction * Math.PI / 2));
      return offset;
   }
   
   public int getAvenue()
   {  return this.avenue;
   }
   
   public int getStreet()
   {  return this.street;
   }
   
   public int getDirection()
   {  return this.direction;
   }
   
	public int distanceToOrigin()
	{	int dist = Math.abs(this.avenue) + Math.abs(this.street);
		return dist;
	}

   public static void main(String[] args)
   {	// Set up a known situation - a robot on intersection (2, 4)
      SimpleCity c = new SimpleCity();
      SimpleBot karel = new SimpleBot(c, 2, 4, EAST);

      // execute the code we want to test
      karel.move();

      // verify the results -- robot on intersection (3,4)
      Test.ckEquals("new ave", 3, karel.getAvenue());
      Test.ckEquals("same str", 4, karel.getStreet());
      Test.ckEquals("same dir", EAST, karel.getDirection());
      
      SimpleBot k = new SimpleBot(c, 1, -1, 1);
      int d = k.distanceToOrigin();
      Test.ckEquals("negative street", 2, d);
   }
}

