
import java.awt.*;
import java.awt.geom.*;
import becker.util.Utilities;


/** A fourth try at the Robot class.  These robots are constructed on an
 * intersection specified by the client, facing East.  Robots can move 
 * forward and turn left.  Robots are shown with the familiar arrowhead 
 * so the user can determine which way they are facing. 
 *
 * @author Byron Weber Becker */
public class SimpleBot4 extends Paintable
{
   private int avenue;
   private int street;
   private int direction;

   /** Construct a new robot in the given city at the given location.
    * @param aCity the city in which this robot appears
    * @param anAvenue this robot's initial avenue
    * @param aStreet this robot's initial street
    * @param dir this robot's initial direction */
   public SimpleBot4(SimpleCity c, int anAvenue, int aStreet, int dir)
   {  super();
      this.avenue = anAvenue;
      this.street = aStreet;
      this.direction = dir;
      c.add(this, 2);
   }

   /** Paint the robot at its current location. */
   public void paint(Graphics2D g)
   {  g.setColor(Color.black);

      int centerX;		// x coordinate of the intersection's center
      int centerY;		// y coordinate of the intersection's center
      // draw the robot's body
      centerX = this.avenue * 50 + 50 / 2;
      centerY = this.street * 50 + 50 / 2;
      g.fillOval(centerX - 15, centerY - 15, 2 * 15, 2 * 15);
      // draw the robot's sensor
      centerX = centerX + this.aveOffset() * 15;
      centerY = centerY + this.strOffset() * 15;
      g.fillOval(centerX - 5, centerY - 5, 2 * 5, 2 * 5);
   }

   /** Move the robot forward 1 intersection. */
   public void move()
   {  this.avenue = this.avenue + this.aveOffset();
      this.street = this.street + this.strOffset();
      Utilities.sleep(400);
   }

   /** Turn the robot left 1/4 turn. */
   public void turnLeft()
   {  this.direction = this.direction + 3;
      Utilities.sleep(400);
   }

   /** Calculate how far the robot should move along the avenue
    @return {-1, 0, or 1} */
   private int aveOffset()
   {  int offset = (int) (Math.cos(this.direction * Math.PI / 2));
      return offset;
   }

   /** Calculate how far the robot should move along the street
    @return {-1, 0, or 1} */
   private int strOffset()
   {  int offset = (int) (Math.sin(this.direction * Math.PI / 2));
      return offset;
   }

}

