import javax.swing.*;
import java.awt.*;

/** A Thermometer component to use in Graphical User Interfaces.
    It can display temperaturs from 0 to 50, inclusive.
    @author Byron Weber Becker */
public class Thermometer extends JComponent
{

   private static final int MIN_TEMP = 0;
   private static final int MAX_TEMP = 50;
   private int temp = MIN_TEMP;

   /** Construct a new Thermometer. */
   public Thermometer()
   {  super();
      Dimension prefSize = new Dimension(30, 250);
      this.setPreferredSize(prefSize);
   }

   /** Paint the thermometer to show the current temperature. */
   public void paintComponent(Graphics g)
   {  super.paintComponent(g);

      int w = this.getWidth();
      int h = this.getHeight();
      int bulbDia = Math.min(w, (int)(h*0.10));
      int bulbTop = h - bulbDia;
      int bulbLeft = w/2-bulbDia/2;
      int mercWidth = bulbDia/3;
      int maxMercHeight = h - bulbDia;
      int mercHeight = (int)(maxMercHeight*(this.temp - MIN_TEMP)/(MAX_TEMP - MIN_TEMP));
      int left = w/2 - mercWidth/2;

      g.setColor(Color.black);
      g.fillOval(bulbLeft, bulbTop, bulbDia, bulbDia);
      g.fillRect(left, 0, mercWidth, bulbTop+5);

      g.setColor(Color.red);
      g.fillRect(left+1, bulbTop-mercHeight+1, mercWidth-2, mercHeight+5);
      g.fillOval(bulbLeft+1, bulbTop+1, bulbDia-2, bulbDia-2);

   }

   /** Set the thermometer's temperature.
    *  @newTemp The new temperature. */
   public void setTemperature(int newTemp)
   {  this.temp = newTemp;
      this.repaint();
   }

   /** Get the thermometer's current temperature.
    *  @returns The thermometer's current temperature. */
   public int getTemperature()
   {  return this.temp;
   }
}