import becker.util.Test;
import becker.xtras.gasPump.IMeter;
	
/** Measure the volume of fuel sold and calculate the amount owed by the
 * customer, given the current fuel cost.
 *
 * @author Byron Weber Becker */
public class Meter extends Object implements IMeter
{
   private double uCost;			// unit cost
   private double volSold = 0.0;	// volume sold
   private int octane;				// octane rating
   private String label;			// marketing label
		
   /** Construct a new Meter object.
    *	@param unitCost The cost for one unit (litre or gallon) of gas 
    * @param octaneRating An integer related to the "performance" of the fuel;  usually 
	 * between 87 and 93.
	 * @param theLabel A label for the fuel such as "Gold" or "Ultra". */
   public Meter(double unitCost, int octaneRating, String theLabel)
   {  super();
      this.uCost = unitCost;
      this.octane = octaneRating;
      this.label = theLabel;
   }	
		
   /** Get the cost per unit of fuel.
    * 	@return cost per unit of fuel */
   public double getUnitCost()
   {  return this.uCost;
   }
	
   /** Get the octane rating of the fuel.
    * @return octane rating (typically between 87 and 93) */
   public int getOctane()
   {  return this.octane;
   }
   
   /** Get the label for this meter's fuel.  For example, "Gold" or "Ultra".
    * @return this meter's fuel label */
   public String getLabel()
   {  return this.label;
   }
	
   /** Pump some fuel into a tank.  This method is called
    * repeatedly while the "handle" on the pump is pressed.
    * @param howMuch How much fuel was pumped since the last time this 
    * method was called. */
   public void pump(double howMuch)
   {  this.volSold = this.volSold + howMuch;
   }
	
   /** Get the volume of fuel sold to this customer.
    * @return volume of fuel sold */
   public double getVolumeSold()
   {  return this.volSold;
   }	
	
   /** Calculate the total cost of fuel sold to this customer.
    * @return price/unit * number of units sold */
   public double calcTotalCost()
   {  double tCost = this.uCost * this.volSold;
      return tCost;
   }
		
   /** Reset the meter for a new customer. */
   public void reset()
   {  this.volSold = 0;
   }
		
   // Test the class.
   public static void main(String[] args)
   {  Meter m1 = new Meter(2.00, 87, "Regular");
      Test.ckEquals("unit cost", 2.00, m1.getUnitCost());
      Meter m2 = new Meter(1.99, 87, "Regular");
      Test.ckEquals("unit cost", 1.99, m2.getUnitCost());
      Test.ckEquals("octane", 87, m2.getOctane());
      Test.ckEquals("label", "Regular", m2.getLabel());
      
      Meter m3 = new Meter(2.00, 87, "Regular");
      Test.ckEquals("volSold", 0.0, m3.getVolumeSold());
      m3.pump(0.02);
      m3.pump(0.03);
      m3.pump(0.01);
      Test.ckEquals("volSold", 0.06, m3.getVolumeSold());
      Test.ckEquals("totCost", 0.06*2.00, m3.calcTotalCost());
      m3.reset();
      Test.ckEquals("after reset", 0.0, m3.getVolumeSold());
      Test.ckEquals("after reset", 0.0, m3.calcTotalCost());
   }
}
